@@ -8,18 +8,20 @@
 
 // spell-checker:ignore (chrono) Datelike Timelike ; (format) DATEFILE MMDDhhmm ; (vars) datetime datetimes
 
-#[macro_use]
-extern crate uucore;
-
 use chrono::{DateTime, FixedOffset, Local, Offset, Utc};
 #[cfg(windows)]
 use chrono::{Datelike, Timelike};
 use clap::{crate_version, App, Arg};
-#[cfg(all(unix, not(target_os = "macos")))]
+#[cfg(all(unix, not(target_os = "macos"), not(target_os = "redox")))]
 use libc::{clock_settime, timespec, CLOCK_REALTIME};
 use std::fs::File;
 use std::io::{BufRead, BufReader};
 use std::path::PathBuf;
+use uucore::display::Quotable;
+#[cfg(not(any(target_os = "macos", target_os = "redox")))]
+use uucore::error::FromIo;
+use uucore::error::{UResult, USimpleError};
+use uucore::show_error;
 #[cfg(windows)]
 use winapi::{
     shared::minwindef::WORD,
@@ -67,10 +69,12 @@ static RFC_3339_HELP_STRING: &str = "output date/time in RFC 3339 format.
  for date and time to the indicated precision.
  Example: 2006-08-14 02:34:56-06:00";
 
-#[cfg(not(target_os = "macos"))]
+#[cfg(not(any(target_os = "macos", target_os = "redox")))]
 static OPT_SET_HELP_STRING: &str = "set time described by STRING";
 #[cfg(target_os = "macos")]
 static OPT_SET_HELP_STRING: &str = "set time described by STRING (not available on mac yet)";
+#[cfg(target_os = "redox")]
+static OPT_SET_HELP_STRING: &str = "set time described by STRING (not available on redox yet)";
 
 /// Settings for this program, parsed from the command line
 struct Settings {
@@ -136,7 +140,8 @@ impl<'a> From<&'a str> for Rfc3339Format {
     }
 }
 
-pub fn uumain(args: impl uucore::Args) -> i32 {
+#[uucore_procs::gen_uumain]
+pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let syntax = format!(
         "{0} [OPTION]... [+FORMAT]...
  {0} [OPTION]... [MMDDhhmm[[CC]YY][.ss]]",
@@ -146,8 +151,10 @@ pub fn uumain(args: impl uucore::Args) -> i32 {
 
     let format = if let Some(form) = matches.value_of(OPT_FORMAT) {
         if !form.starts_with('+') {
-            eprintln!("date: invalid date '{}'", form);
-            return 1;
+            return Err(USimpleError::new(
+                1,
+                format!("invalid date {}", form.quote()),
+            ));
         }
         let form = form[1..].to_string();
         Format::Custom(form)
@@ -175,8 +182,10 @@ pub fn uumain(args: impl uucore::Args) -> i32 {
     let set_to = match matches.value_of(OPT_SET).map(parse_date) {
         None => None,
         Some(Err((input, _err))) => {
-            eprintln!("date: invalid date '{}'", input);
-            return 1;
+            return Err(USimpleError::new(
+                1,
+                format!("invalid date {}", input.quote()),
+            ));
         }
         Some(Ok(date)) => Some(date),
     };
@@ -240,18 +249,16 @@ pub fn uumain(args: impl uucore::Args) -> i32 {
                     let formatted = date.format(format_string).to_string().replace("%f", "%N");
                     println!("{}", formatted);
                 }
-                Err((input, _err)) => {
-                    println!("date: invalid date '{}'", input);
-                }
+                Err((input, _err)) => show_error!("invalid date {}", input.quote()),
             }
         }
     }
 
-    0
+    Ok(())
 }
 
 pub fn uu_app() -> App<'static, 'static> {
-    App::new(executable!())
+    App::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
         .arg(
@@ -347,23 +354,33 @@ fn parse_date<S: AsRef<str> + Clone>(
 }
 
 #[cfg(not(any(unix, windows)))]
-fn set_system_datetime(_date: DateTime<Utc>) -> i32 {
+fn set_system_datetime(_date: DateTime<Utc>) -> UResult<()> {
     unimplemented!("setting date not implemented (unsupported target)");
 }
 
 #[cfg(target_os = "macos")]
-fn set_system_datetime(_date: DateTime<Utc>) -> i32 {
-    eprintln!("date: setting the date is not supported by macOS");
-    1
+fn set_system_datetime(_date: DateTime<Utc>) -> UResult<()> {
+    Err(USimpleError::new(
+        1,
+        "setting the date is not supported by macOS".to_string(),
+    ))
 }
 
-#[cfg(all(unix, not(target_os = "macos")))]
+#[cfg(target_os = "redox")]
+fn set_system_datetime(_date: DateTime<Utc>) -> UResult<()> {
+    Err(USimpleError::new(
+        1,
+        "setting the date is not supported by Redox".to_string(),
+    ))
+}
+
+#[cfg(all(unix, not(target_os = "macos"), not(target_os = "redox")))]
 /// System call to set date (unix).
 /// See here for more:
 /// https://doc.rust-lang.org/libc/i686-unknown-linux-gnu/libc/fn.clock_settime.html
 /// https://linux.die.net/man/3/clock_settime
 /// https://www.gnu.org/software/libc/manual/html_node/Time-Types.html
-fn set_system_datetime(date: DateTime<Utc>) -> i32 {
+fn set_system_datetime(date: DateTime<Utc>) -> UResult<()> {
     let timespec = timespec {
         tv_sec: date.timestamp() as _,
         tv_nsec: date.timestamp_subsec_nanos() as _,
@@ -372,11 +389,9 @@ fn set_system_datetime(date: DateTime<Utc>) -> i32 {
     let result = unsafe { clock_settime(CLOCK_REALTIME, &timespec) };
 
     if result != 0 {
-        let error = std::io::Error::last_os_error();
-        eprintln!("date: cannot set date: {}", error);
-        error.raw_os_error().unwrap()
+        Err(std::io::Error::last_os_error().map_err_context(|| "cannot set date".to_string()))
     } else {
-        0
+        Ok(())
     }
 }
 
@@ -385,7 +400,7 @@ fn set_system_datetime(date: DateTime<Utc>) -> i32 {
 /// See here for more:
 /// https://docs.microsoft.com/en-us/windows/win32/api/sysinfoapi/nf-sysinfoapi-setsystemtime
 /// https://docs.microsoft.com/en-us/windows/win32/api/minwinbase/ns-minwinbase-systemtime
-fn set_system_datetime(date: DateTime<Utc>) -> i32 {
+fn set_system_datetime(date: DateTime<Utc>) -> UResult<()> {
     let system_time = SYSTEMTIME {
         wYear: date.year() as WORD,
         wMonth: date.month() as WORD,
@@ -402,10 +417,8 @@ fn set_system_datetime(date: DateTime<Utc>) -> i32 {
     let result = unsafe { SetSystemTime(&system_time) };
 
     if result == 0 {
-        let error = std::io::Error::last_os_error();
-        eprintln!("date: cannot set date: {}", error);
-        error.raw_os_error().unwrap()
+        Err(std::io::Error::last_os_error().map_err_context(|| "cannot set date".to_string()))
     } else {
-        0
+        Ok(())
     }
 }
