@@ -7,12 +7,6 @@
 
 // spell-checker:ignore (ToDO) gethostid
 
-// Clippy bug: https://github.com/rust-lang/rust-clippy/issues/7422
-#![allow(clippy::nonstandard_macro_braces)]
-
-#[macro_use]
-extern crate uucore;
-
 use clap::{crate_version, App};
 use libc::c_long;
 use uucore::error::UResult;
@@ -32,7 +26,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 }
 
 pub fn uu_app() -> App<'static, 'static> {
-    App::new(executable!())
+    App::new(uucore::util_name())
         .version(crate_version!())
         .usage(SYNTAX)
 }
