@@ -7,7 +7,9 @@
 
 #[macro_use]
 extern crate uucore;
+use uucore::display::Quotable;
 use uucore::entries;
+use uucore::error::{UResult, USimpleError};
 use uucore::fs::display_permissions;
 use uucore::fsext::{
     pretty_filetype, pretty_fstype, pretty_time, read_fs_list, statfs, BirthTime, FsMeta,
@@ -24,7 +26,10 @@ use std::{cmp, fs, iter};
 macro_rules! check_bound {
     ($str: ident, $bound:expr, $beg: expr, $end: expr) => {
         if $end >= $bound {
-            return Err(format!("'{}': invalid directive", &$str[$beg..$end]));
+            return Err(USimpleError::new(
+                1,
+                format!("{}: invalid directive", $str[$beg..$end].quote()),
+            ));
         }
     };
 }
@@ -331,7 +336,7 @@ fn print_it(arg: &str, output_type: OutputType, flag: u8, width: usize, precisio
 }
 
 impl Stater {
-    pub fn generate_tokens(format_str: &str, use_printf: bool) -> Result<Vec<Token>, String> {
+    pub fn generate_tokens(format_str: &str, use_printf: bool) -> UResult<Vec<Token>> {
         let mut tokens = Vec::new();
         let bound = format_str.len();
         let chars = format_str.chars().collect::<Vec<char>>();
@@ -436,6 +441,7 @@ impl Stater {
                             'f' => tokens.push(Token::Char('\x0C')),
                             'n' => tokens.push(Token::Char('\n')),
                             'r' => tokens.push(Token::Char('\r')),
+                            't' => tokens.push(Token::Char('\t')),
                             'v' => tokens.push(Token::Char('\x0B')),
                             c => {
                                 show_warning!("unrecognized escape '\\{}'", c);
@@ -455,7 +461,7 @@ impl Stater {
         Ok(tokens)
     }
 
-    fn new(matches: ArgMatches) -> Result<Stater, String> {
+    fn new(matches: ArgMatches) -> UResult<Stater> {
         let files: Vec<String> = matches
             .values_of(ARG_FILES)
             .map(|v| v.map(ToString::to_string).collect())
@@ -474,14 +480,12 @@ impl Stater {
         let show_fs = matches.is_present(options::FILE_SYSTEM);
 
         let default_tokens = if format_str.is_empty() {
-            Stater::generate_tokens(&Stater::default_format(show_fs, terse, false), use_printf)
-                .unwrap()
+            Stater::generate_tokens(&Stater::default_format(show_fs, terse, false), use_printf)?
         } else {
             Stater::generate_tokens(format_str, use_printf)?
         };
         let default_dev_tokens =
-            Stater::generate_tokens(&Stater::default_format(show_fs, terse, true), use_printf)
-                .unwrap();
+            Stater::generate_tokens(&Stater::default_format(show_fs, terse, true), use_printf)?;
 
         let mount_list = if show_fs {
             // mount points aren't displayed when showing filesystem information
@@ -651,11 +655,7 @@ impl Stater {
                                                     return 1;
                                                 }
                                             };
-                                            arg = format!(
-                                                "`{}' -> `{}'",
-                                                file,
-                                                dst.to_string_lossy()
-                                            );
+                                            arg = format!("{} -> {}", file.quote(), dst.quote());
                                         } else {
                                             arg = file.to_string();
                                         }
@@ -749,7 +749,7 @@ impl Stater {
                     }
                 }
                 Err(e) => {
-                    show_error!("cannot stat '{}': {}", file, e);
+                    show_error!("cannot stat {}: {}", file.quote(), e);
                     return 1;
                 }
             }
@@ -842,7 +842,11 @@ impl Stater {
                     }
                 }
                 Err(e) => {
-                    show_error!("cannot read file system information for '{}': {}", file, e);
+                    show_error!(
+                        "cannot read file system information for {}: {}",
+                        file.quote(),
+                        e
+                    );
                     return 1;
                 }
             }
@@ -881,8 +885,8 @@ impl Stater {
     }
 }
 
-fn get_usage() -> String {
-    format!("{0} [OPTION]... FILE...", executable!())
+fn usage() -> String {
+    format!("{0} [OPTION]... FILE...", uucore::execution_phrase())
 }
 
 fn get_long_usage() -> String {
@@ -943,8 +947,9 @@ for details about the options it supports.
     )
 }
 
-pub fn uumain(args: impl uucore::Args) -> i32 {
-    let usage = get_usage();
+#[uucore_procs::gen_uumain]
+pub fn uumain(args: impl uucore::Args) -> UResult<()> {
+    let usage = usage();
     let long_usage = get_long_usage();
 
     let matches = uu_app()
@@ -952,17 +957,17 @@ pub fn uumain(args: impl uucore::Args) -> i32 {
         .after_help(&long_usage[..])
         .get_matches_from(args);
 
-    match Stater::new(matches) {
-        Ok(stater) => stater.exec(),
-        Err(e) => {
-            show_error!("{}", e);
-            1
-        }
+    let stater = Stater::new(matches)?;
+    let exit_status = stater.exec();
+    if exit_status == 0 {
+        Ok(())
+    } else {
+        Err(exit_status.into())
     }
 }
 
 pub fn uu_app() -> App<'static, 'static> {
-    App::new(executable!())
+    App::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
         .arg(
