@@ -505,3 +505,66 @@ fn test_round() {
             .stdout_only(exp.join("\n") + "\n");
     }
 }
+
+#[test]
+fn test_suffix_is_added_if_not_supplied() {
+    new_ucmd!()
+        .args(&["--suffix=TEST"])
+        .pipe_in("1000")
+        .succeeds()
+        .stdout_only("1000TEST\n");
+}
+
+#[test]
+fn test_suffix_is_preserved() {
+    new_ucmd!()
+        .args(&["--suffix=TEST"])
+        .pipe_in("1000TEST")
+        .succeeds()
+        .stdout_only("1000TEST\n");
+}
+
+#[test]
+fn test_suffix_is_only_applied_to_selected_field() {
+    new_ucmd!()
+        .args(&["--suffix=TEST", "--field=2"])
+        .pipe_in("1000 2000 3000")
+        .succeeds()
+        .stdout_only("1000 2000TEST 3000\n");
+}
+
+#[test]
+fn test_transform_with_suffix_on_input() {
+    new_ucmd!()
+        .args(&["--suffix=b", "--to=si"])
+        .pipe_in("2000b")
+        .succeeds()
+        .stdout_only("2.0Kb\n");
+}
+
+#[test]
+fn test_transform_without_suffix_on_input() {
+    new_ucmd!()
+        .args(&["--suffix=b", "--to=si"])
+        .pipe_in("2000")
+        .succeeds()
+        .stdout_only("2.0Kb\n");
+}
+
+#[test]
+fn test_transform_with_suffix_and_delimiter() {
+    new_ucmd!()
+        .args(&["--suffix=b", "--to=si", "-d=|"])
+        .pipe_in("1000b|2000|3000")
+        .succeeds()
+        .stdout_only("1.0Kb|2000|3000\n");
+}
+
+#[test]
+fn test_suffix_with_padding() {
+    new_ucmd!()
+        .args(&["--suffix=pad", "--padding=12"])
+        .pipe_in("1000 2000 3000")
+        .succeeds()
+        .stdout_only("     1000pad 2000 3000\n");
+}
